package models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import play.*;
import com.sulake.habbo.gaming.api.client.GamingApiClient;
import com.sulake.habbo.gaming.api.domain.LoadGameUrlRequest;
import com.sulake.habbo.gaming.api.domain.Player;


public class Game {

	protected static Game instance;
	
	private List<Match> currentMatches;
	private List<Player> apiPlayers;
	private Map<String,Integer> wins_counter;
	
	GamingApiClient gamingClient;
	
	/**Constructor of the game*/
	protected Game() {
		this.gamingClient = new GamingApiClient("http://s1.varoke.net/habbo-gaming-api-server/v1/numbergame", "LIzJidqPTxJFNgig7lp8kxQ6Rv5thnMM", "vs5NVyxgtogO2gpPmR3JWbem0HLCqZfy");
		apiPlayers = new ArrayList<Player>();
		currentMatches = new ArrayList<Match>();
		this.wins_counter = new HashMap<String, Integer>();
		
	}
	
	/**Return the instance of game, if null, create a new one*/		
	public synchronized static Game getInstance() {
		if (instance==null) {
			instance = new Game();
		}
		return instance;
	}
	
	/**Api call to Habbo server to return to flash client*/
	public void quitGame(String accessToken){
		System.out.println("Player: " +accessToken +" has left the game");
		this.gamingClient.quitGame(accessToken);
		this.deleteUserFromList(accessToken);
		this.wins_counter.remove(accessToken);
	}
	
	/**starts connection to game server*/
	public void startConnection(){
		this.gamingClient = new GamingApiClient("http://s1.varoke.net/habbo-gaming-api-server/v1/numbergame", "LIzJidqPTxJFNgig7lp8kxQ6Rv5thnMM", "vs5NVyxgtogO2gpPmR3JWbem0HLCqZfy");
	}
	
	/**Delete the user from the game queue*/
	private void deleteUserFromList(String aToken){
		for(int a = 0; a < apiPlayers.size(); a++){
			if(apiPlayers.get(a).getAccessToken().equals(aToken)){
				apiPlayers.remove(a);
			}
		}
	}
	
	/**
	 * set the players list from game server*/
	public void addPlayers(){
		List<Player> aux = this.gamingClient.pollPlayers();
		
		for(int a = 0; aux.size() != a; a++  ){
			if( !apiPlayers.contains( aux.get(a) ) ){
				System.out.println("Element: " + aux.get(a).getAccessToken() + " is not in the list.");
				LoadGameUrlRequest urlP1 = new LoadGameUrlRequest("http://dhcp70.sulake.com:9000/play/"+ aux.get(a).getAccessToken() );
				this.gamingClient.loadGameUrl(aux.get(a).getAccessToken(), urlP1);
				this.apiPlayers.add(aux.get(a));
				Logger.info("One player allocated: " + aux.get(a).getAccessToken() );
				System.out.println( "Add player: "+aux.get(a).getAccessToken() + " in position: "+ a);
			}
		}
	}
	
	/**
	 * set the players list from game server*/
	public void addPlayers(List<Player> list ){
		List<Player> aux  = list;
		
		for(int a = 0; aux.size() != a; a++  ){
			if( !apiPlayers.contains( aux.get(a) ) ){
				System.out.println("Element: " + aux.get(a).getAccessToken() + " is not in the list.");
				this.apiPlayers.add(aux.get(a));
				System.out.println( "Add player: "+aux.get(a).getAccessToken() + " in position: "+ a);
			}
		}
	}
	
	/*
	 * Start the match linking two players from the list*/
	public void start_match(){		
		while( apiPlayers.size() > 1){
			Player p1 = apiPlayers.remove(0) ;
			Player p2 = apiPlayers.remove(0);
			this.currentMatches.add(new Match(p1,p2));
		}
	}
	
	/**
	 * @param String access token of the player
	 * @return the Match where the player is playing*/
	public Match getMatchForPlayer(String accessToken) {
		for(Match currentMatch : currentMatches) {
			if (currentMatch.getPlayer1().getAtoken().equals(accessToken)) {
				return currentMatch;
			} else if(currentMatch.getPlayer2().getAtoken().equals(accessToken)) {
				return currentMatch;
			}
		}
		return null;
	}
	
	/**
	 * @param String Access Token
	 * Ends the previous game and relocate the winner into the list*/
	public void relocateWinner(String accessToken){		
		Match cMatch = this.getMatchForPlayer(accessToken);
		Players winner = cMatch.getWinner();

		this.currentMatches.remove(cMatch);
		Player new_player = new Player();
		new_player.setAccessToken(winner.getAtoken());
		new_player.setPlayerName(winner.getName());
		this.add_win(accessToken);
		this.apiPlayers.add(new_player);
	}
		
	/**
	 * @return int size of the players list
	 * method that returns the size of the players*/
	public int getSize(){
		return apiPlayers.size();
	}
	
	/**@param String access token
	 * Add one to the wins counter to each player*/
	public void add_win(String aToken){
		Integer wins = 0;
		if(this.wins_counter.containsKey(aToken) ){
			wins = this.wins_counter.get(aToken);
			wins++;
			this.wins_counter.put(aToken, wins);
		} else {
			this.wins_counter.put(aToken, wins);
		}
	}

}//EOClass Game
