package models;

import java.util.ArrayList;
import java.util.List;
import com.sulake.habbo.gaming.api.domain.InventoryItem;
import com.sulake.habbo.gaming.api.domain.Offer;
import com.sulake.habbo.gaming.api.domain.Player;
import com.sulake.habbo.gaming.api.domain.PlayerImageRequest;
import com.sulake.habbo.gaming.api.domain.PlayerImageRequest.Direction;
import com.sulake.habbo.gaming.api.domain.PlayerImageRequest.Size;


/**
 * @author Sulake 2012
 * Match class that controls the game between two players
 */
public class Match extends Game  {

	private Players 	player1;
	private Players 	player2;
	private int			min_range;
	private int 		max_range;
	private PowerUps	p_ups;	
	private Offer		cut_the_range;
	private Offer		half_the_range;
	
	/*
	 * @param Player 1
	 * @param Player 2
	 * method that creates the environment for the game*/	
	public Match(Player p1, Player p2){
		super();
		this.setRange();
		this.p_ups = new PowerUps(1,10);
		this.player1 = new Players( p1.getPlayerName(), p1.getAccessToken(), true, false, true );
		this.player2 = new Players( p2.getPlayerName(), p2.getAccessToken(), false, false, false );
		getImages();
		getOffers();
		updatePlayersInventory();
	}
	
	/**@return return the minimun value for the range*/
	public int getMinRange(){
		return this.min_range;
	}
	 
	/**@return the maximum value for the range*/
	public int getMaxRange(){
		return this.max_range;
	}
	
	/*
	 * @return Player with the access token
	 * @param String Access Token */
	public Players getPlayer(String aToken){
		if(this.player1.getAtoken().equals(aToken)) { return this.player1; }
		else { return this.player2;}
	}
	
	/**@return the player 1*/
	public Players getPlayer1() {
		return player1;
	}

	/**@return The player 2*/
	public Players getPlayer2() {
		return player2;
	}
	
	/**Method that sets the min and max range [1,10]*/
	private void setRange(){
		this.max_range = 10;
		this.min_range = 1;
	}
	
	/*
	 * @return returns the access token of the first player in game*/
	public String whoIsFirst(){
		if(this.player1.isFirst()){ 	return this.player1.getAtoken(); }
		else {							return this.player2.getAtoken(); }
	}
	
	/** @return the name of the first player*/
	public String whoIsFirstName(){
		if(this.player1.isFirst()){ 	return this.player1.getName(); }
		else {							return this.player2.getName(); }
	}
	
	/**Get the avatar images for each player*/
	private void getImages(){
		List<PlayerImageRequest> list_img = new ArrayList<PlayerImageRequest>();
		PlayerImageRequest aux = new PlayerImageRequest();
		aux.setDirection(Direction.S);
		aux.setSize(Size.SIZE64);
		aux.setHeadDirection(Direction.S);
		aux.setHeadOnly(false);
		aux.setWithoutHatAndHair(false);
		list_img.add(aux);
		
		List<String> img_url;
		try{
			img_url = this.gamingClient.getImageUrls(this.player1.getAtoken(), list_img);
			if(img_url.size()==0){ 
				this.player1.setAvatar_url("http://www.habbo.com/habbo-imaging/avatar/hr-515-33.hd-600-1.ch-635-70.lg-716-66-62.sh-735-68.ha-1024,s-2.g-0.d-3.h-3.a-0,672a99ef51ce41fb7e66336ff2c724d3.png");
			} else {
				System.out.println(img_url.size());
				this.player1.setAvatar_url(img_url.get(0));
			}
		
			img_url = this.gamingClient.getImageUrls(this.player2.getAtoken(), list_img);
		
			if(img_url.size()==0){ 
				this.player2.setAvatar_url("http://www.habbo.com/habbo-imaging/avatar/hr-515-33.hd-600-1.ch-635-70.lg-716-66-62.sh-735-68.ha-1024,s-2.g-0.d-3.h-3.a-0,672a99ef51ce41fb7e66336ff2c724d3.png");
			} else {
				this.player2.setAvatar_url(img_url.get(0));	
			}
		}catch (Exception a) {
			this.quitGame(this.player1.getAtoken());
			this.quitGame(this.player2.getAtoken());
		};
	}

	/**@return access token of the winner*/
	public String whoWins(){
		if(this.player1.isWinner()){
			return this.player1.getAtoken();
		}
		if(this.player2.isWinner()) { 
			return this.player2.getAtoken();
		} else {
			return "HabboWins";
		}
		
	}

	/** change turn order*/
	private void changeTurn(){
		boolean aux = this.player1.isMyTurn();
		this.player1.setMyTurn(this.player2.isMyTurn());
		this.player2.setMyTurn(aux);
	}
	
	/**
	 * @param integer number 
	 * @param String access token
	 * set the number that a user introduce*/
	public void setNumber(int number, String aToken){
		if(this.player1.getAtoken().equals(aToken)){
			this.player1.setNumber(number);
			this.p_ups.setNumber(number);
			this.changeTurn();
		}else {
			this.player2.setNumber(number);
			if(this.player1.getNumber() == this.player2.getNumber()) { this.changeTurn(); }
			if( this.player2.getTries() == 1) { this.changeTurn(); }
			this.player2.decreaseTry();
		}
	}
	
	/**@return the number of tries are left for player 2*/
	public int getTries(){
		return this.player2.getTries();
	}
	
	
	/*
	 * @return return if is the player's turn */
	public boolean myTurn(String aToken){
		if(this.player1.getAtoken().equals(aToken)){
			return this.player1.isMyTurn();
		}else {
			return this.player2.isMyTurn();
		}	
	}
	
	/**
	 * @return map including the access token and boolean true/false winner/loser*/
	public void checkWinner(){
		if( this.player1.compareTo(this.player2) == 1){
			this.player2.add_win();
			this.player2.setWinner(true);
			this.player2.setTries();
		} else {
			if( this.getTries() == 0 ) {
				this.player1.add_win();
				this.player1.setWinner(true);
			}
		}
	}
	
	/*
	 * @return Players return the player who won the game otherwise null*/
	public Players getWinner(){
		if( this.player1.getAtoken().equals( this.whoWins() ) ){
			return this.player1;
		} 
		if (this.player2.getAtoken().equals(this.whoWins()) ) {
			return this.player2;
		} else { return null; }
	}
	
	/**@param amount of Half the user wants to buy*/
	public void buyHalf(int amount){
		for (int i = 0; i<amount; i++){
			this.gamingClient.purchaseOffer(this.player2.getAtoken(), this.half_the_range.getId());
		}
		this.player2.buyHalf(amount);
	}
	
	/**@param amount of Cut the user wants to buy*/
	public void buyCut(int amount){
		for(int i = 0; i<amount; i++){
			this.gamingClient.purchaseOffer(this.player2.getAtoken(), this.cut_the_range.getId());
		}
		this.player2.buyCut(amount);
	}
	
	/**Execute the Half range and update the range*/
	public void halfRange(){
		if(this.player2.useHalf()){
			this.p_ups.setNumber(this.player1.getNumber());	
			this.gamingClient.removeFromInventory(this.player2.getAtoken(), this.half_the_range.getItemType(), 1);
			this.p_ups.half_range();
			this.updateRange();
		}
	}
	
	/**Execute the Cut range and update the range*/
	public void cutRange(){
		if(this.player2.useCut()){
			this.gamingClient.removeFromInventory(this.player2.getAtoken(), this.cut_the_range.getItemType(),1);
			this.p_ups.cut_range();
			this.updateRange();
		}
	}
	
	/**Update the max and min values of the range inside the PowerUps class*/
	private void updateRange(){
		this.max_range = this.p_ups.getMax();
		this.min_range = this.p_ups.getMin();
	}
	
	/**@param integer number, check if the intruced number is within the range*/
	public boolean inRange(int num){
		if ( (num <= this.max_range) && (num >= this.min_range)) {
			return true;
		} else { return false;}
	}
	
	/**@return amount of cut the range left*/
	public int getCut(){
		return this.player2.getNum_cut_range();
	}
	
	
	/**@param Amount of half the range left*/
	public int getHalf(){
		return this.player2.getNum_half_range();
	}
	
	/**Update the player inventory by getting it from the API call*/
	private void updatePlayersInventory(){
		List<InventoryItem> aux1 = this.gamingClient.getInventory(this.player2.getAtoken());
		for(int i = 0 ; i < aux1.size(); i++){
			InventoryItem item1 = aux1.get(i);
			if(item1.getItemType().equals("CUT")){ 	this.player2.setNum_cut_range(item1.getCount()); }
			else { 									this.player2.setNum_half_range(item1.getCount()); }
		}
	}
		
	/**@return the amount of the Purse*/
	public int getPurse(){
		return  this.gamingClient.getPurse(this.player2.getAtoken()).getCredits();
	}
	
	/**Get the list of offers are set in the game server*/
	private void getOffers(){
		List<Offer> offers = this.gamingClient.getOffers();
		this.cut_the_range = offers.get(0);
		this.half_the_range= offers.get(1);	
	}
	
}//end class Match
